//canvas.cpp



#include <gnome.h>
#include "theme.h"
#include "widget.h"
#include "cursor.h"
#include "canvas.h"



CanvasItem::CanvasItem()
{
}

CanvasItem::~CanvasItem()
{
	gtk_object_destroy(GTK_OBJECT(_item));
}

CanvasItem::CanvasItem(GnomeCanvasItem* item)
{
	_item = item;
}


//useful
void CanvasItem::w2i(double* x, double* y)
{
//	gnome_canvas_item_w2i(_item->parent, x, y);
	gnome_canvas_item_w2i(_item, x, y);
}

void CanvasItem::grab(unsigned int event_mask, Cursor* cursor, guint32 etime)
{
	gnome_canvas_item_grab(_item, event_mask, cursor->cursor(), etime);
}

void CanvasItem::ungrab(guint32 etime)
{
	gnome_canvas_item_ungrab(_item, etime);
}

void CanvasItem::move(double x, double y)
{
	gnome_canvas_item_move(_item, x, y);
}

void CanvasItem::signalConnect(void* func, gpointer data)
{
	gtk_signal_connect(GTK_OBJECT(GNOME_CANVAS_ITEM(_item)), "event", GTK_SIGNAL_FUNC(func), data);
}

void CanvasItem::show()
{
	gnome_canvas_item_show(_item);
}

void CanvasItem::hide()
{
	gnome_canvas_item_hide(_item);
}

void CanvasItem::redraw() //FIXME
{
//	gnome_canvas_item_request_redraw_svp(_item);
//	gnome_canvas_item_request_update(_item);
}

void CanvasItem::lower(gint n = 0)
{
	if(n) gnome_canvas_item_lower(_item, n);
	else gnome_canvas_item_lower_to_bottom(_item);
}

void CanvasItem::raise(gint n = 0)
{
	if(n) gnome_canvas_item_raise(_item, n);
	else gnome_canvas_item_raise_to_top(_item);
}


CanvasGroup::CanvasGroup(Canvas* canvas)
{
	_item = gnome_canvas_item_new(GNOME_CANVAS_GROUP(canvas->root()),
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL);
}

CanvasGroup::CanvasGroup(GnomeCanvasItem* group)
{
	_item = group;
}


//create objects
CanvasItemLine::CanvasItemLine(CanvasGroup* group, double x1, double y1, double x2, double y2, double width): CanvasItem()
{
	GnomeCanvasPoints* points;

	//we must create a points list
	points = gnome_canvas_points_new(2);
	points->coords[0] = x1;
	points->coords[1] = y1;
	points->coords[2] = x2;
	points->coords[3] = y2;

	_item = gnome_canvas_item_new(group->group(),
			gnome_canvas_line_get_type(),
			"points", points,
			"fill_color", "black",
			"width_units", width,
			NULL);

	//free the points list and return
	gnome_canvas_points_free(points);
}

//sets
void CanvasItemLine::setCoords(double x1, double y1, double x2, double y2)
{
	GnomeCanvasPoints* points;

	//create a points list
	points = gnome_canvas_points_new(2);
	points->coords[0] = x1;
	points->coords[1] = y1;
	points->coords[2] = x2;
	points->coords[3] = y2;

	gnome_canvas_item_set(_item,
			"points", points,
			NULL);

	//free the points list and return
	gnome_canvas_points_free(points);
}

void CanvasItemLine::setArrows(gboolean start = FALSE, gboolean end = FALSE,
		double width = 3.0, double length = 10.0, double edge = 10.0)
{
	gnome_canvas_item_set(_item,
			"first_arrowhead", start,
			"last_arrowhead", end,
			"arrow_shape_a", length,
			"arrow_shape_b", edge,
			"arrow_shape_c", width,
			NULL);
}

CanvasItemRect::CanvasItemRect(CanvasGroup* group, double x1, double y1, double x2, double y2, double width): CanvasItem()
{
	_item = gnome_canvas_item_new(group->group(),
		gnome_canvas_rect_get_type(),
		"x1", x1, "y1", y1,
		"x2", x2, "y2", y2,
		"fill_color", "white",
		"outline_color", "black",
		"width_units", width,
		NULL);
}

//returns
void CanvasItemRect::get(double* x1, double* y1, double* x2, double* y2)
{
	gnome_canvas_item_get_bounds(_item, x1, y1, x2, y2);
}

//sets
void CanvasItemRect::set(double x1, double y1, double x2, double y2, double width)
{
	gnome_canvas_item_set(_item,
			"x1", x1, "y1", y1,
			"x2", x2, "y2", y2,
			"width_units", width,
			NULL);
}

CanvasItemText::CanvasItemText(CanvasGroup* group, double x, double y, char text[]): CanvasItem()
{
	_x = x; _y = y;
	_group = group;
	_item = gnome_canvas_item_new(group->group(),
		gnome_canvas_text_get_type(),
		"x", x,
		"y", y,
		"anchor", GTK_ANCHOR_WEST,
		"font", "fixed",
		"fill_color", "black",
		"text", text,
		NULL);
}

void CanvasItemText::setText(gchar text[]) //FIXME solution bourrin
{
	gtk_object_destroy(GTK_OBJECT(_item));
	_item = gnome_canvas_item_new(_group->group(),
			gnome_canvas_text_get_type(),
			"x", _x,
			"y", _y,
			"anchor", GTK_ANCHOR_WEST,
			"font", "fixed",
			"fill_color", "black",
			"text", text,
			NULL);
}

//Canvas class
Canvas::Canvas(double scroll_x, double scroll_y, char bg_color[]): Widget()
{
#ifndef CANVAS_AA	//no anti-aliasing
	gtk_widget_push_visual(gdk_rgb_get_visual());
	gtk_widget_push_colormap(gdk_rgb_get_cmap());
	_widget = gnome_canvas_new();
	gtk_widget_pop_colormap();
	gtk_widget_pop_visual();
#else			//anti-aliasing
	gtk_widget_push_visual(gdk_imlib_get_visual());
	gtk_widget_push_colormap(gdk_imlib_get_colormap());
	_widget = gnome_canvas_new_aa();
	gtk_widget_pop_visual();
	gtk_widget_pop_colormap();
#endif
	gnome_canvas_set_scroll_region(GNOME_CANVAS(_widget), 0, 0, scroll_x, scroll_y);

/*	if(bg_color)
	{
		gnome_canvas_item_new(gnome_canvas_root(GNOME_CANVAS(_widget)),
				gnome_canvas_rect_get_type(),
				"x1", 0.0, "y1", 0.0,
				"x2", scroll_x, "y2", scroll_y,
				"fill_color", bg_color,
				NULL);
	}*/
}

//sets
void Canvas::setScale(double s = 1.0)
{
	gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(_widget), s);
}


//create a group
CanvasGroup* Canvas::newGroup()
{
	return new CanvasGroup(gnome_canvas_item_new(root(),
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL));
}

//coordonates
void Canvas::c2w(int cx, int cy, double* x, double* y)
{
	gnome_canvas_c2w(GNOME_CANVAS(_widget), cx, cy, x, y);
}

//useful
void Canvas::redraw()
{
	gnome_canvas_update_now(GNOME_CANVAS(_widget));
}

void Canvas::redraw(gint x1, gint y1, gint x2, gint y2)
{
	//redraws from (x1,y1) to (x2-1,y2-1)
	gnome_canvas_request_redraw(GNOME_CANVAS(_widget), x1, y1, ++x2, ++y2);
}
